from cleantext import clean as _clean


from legal_doc_processing import logger


def _del_dummy_breaklines(txt: str):
    """ """

    new_txt = (
        txt.replace("\n.", "$$$$")
        .replace("\n", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("$$$$", "\n")
    )

    return new_txt


def _transform_double_breaks(txt: str, sep="\n----\n") -> str:
    """ """

    new_txt = (
        txt.replace("\n\n", sep)
        .replace("\n\n", sep)
        .replace("\n\n", sep)
        .replace("\n\n", sep)
        .replace("\n\n", sep)
    )

    return new_txt


def _del_double_spaces(txt: str) -> str:
    """ """

    new_txt = (
        txt.replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
    )

    return new_txt


def _del_bouble_breaks(txt: str) -> str:
    """ """

    new_txt = (
        txt.replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
    )

    return new_txt


def _detect_chapter(txt: str, threshold: int = 5, sep="----") -> str:
    """ """

    txt = txt.splitlines()

    cand_chapter = lambda i: i if ((len(i) >= threshold) or (i == sep)) else "!!!!\n" + i
    txt = [cand_chapter(i) for i in txt]

    return "\n".join(txt)


def _del_double_section(txt, sep="\n----\n"):
    """ """

    new_txt = (
        txt.replace(sep + sep, sep)
        .replace(sep + sep, sep)
        .replace(sep + sep, sep)
        .replace(sep + sep, sep)
        .replace(sep + sep, sep)
    )

    return new_txt


def _strip(txt: str) -> str:
    """ """

    txt = txt.splitlines()
    txt = [i.strip() for i in txt]

    return "\n".join(txt)


def _del_bouble_breaks_and_spaces(txt: str) -> str:
    """ """

    new_txt = (
        txt.replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
        .replace("\n\n", "\n")
    )

    return new_txt


def _split_pages(txt: str) -> list:
    """ """

    if "\x0c" in txt:
        pages = txt.split("\x0c")
    else:
        pages = [
            txt,
        ]

    return pages


def _split_lines_pages(pages: list) -> list:
    """ """

    return [i.split("\n") for i in pages]


def _detect_true_text_id(first_page: str, line_length_txt=60, n_lines=4) -> int:
    """ """

    first_page = first_page.splitlines()

    # find a first candidate for a REAL senetence
    # i, len char the line, the line itself
    cands_1st = [(i, len(j), j) for i, j in enumerate(first_page)]

    # then we want the len of this line, the len of the n+1 line, idem n+2, n+3
    cands_2nd = [
        (
            i,
            [cands_1st[i + k][1] for k in range(n_lines)],
            k,
        )
        for i, j, k in cands_1st[: -(n_lines + 1)]
    ]

    # we want to know if the 3 next lines will be len() > threshold
    cands_3rd = [(i, sum([kk > line_length_txt for kk in j]), k) for i, j, k in cands_2nd]

    # if not all then True
    cands_5th = [(i, j, k) for i, j, k in cands_3rd if j >= n_lines]

    if not len(cands_5th):
        return -1
    else:
        return cands_5th[0][0]


def _do_split_header_page(txt, _id):
    """ """

    txt = txt.splitlines()

    header = txt[:_id]
    page = txt[_id:]

    return "\n".join(header), "\n".join(page)


def _ultimate_clean(txt: str) -> str:
    """ """

    txt = txt.splitlines()
    txt = [i.strip() for i in txt]
    txt = [i for i in txt if (i and (i != ")"))]
    txt = "\n".join(txt)

    return txt


def _transfert_title_from_head_to_page(header: str, page: str) -> tuple:
    """ """

    last_title = False

    one_of_in = lambda i: any([char in i for char in list("1il.,")])
    if len(header[-1]) <= 3 and one_of_in(header[-1].lower()):
        last_title = True

    if "introduc" in header[-1].lower():
        last_title = True

    if not last_title:
        return header, page

    # transfert last line to 1st line
    header, page = header.splitlines(), page.splitlines()
    title = header[-1]
    header = header[:-1]
    page = [
        title,
    ] + page

    header = "\n".join(header)
    page = "\n".join(page)

    return header, page


def structure_legal_doc(
    raw_txt, juridiction="", nlspa=None, line_length_txt=50, n_lines=5
):
    """ """

    # 1st split by pages
    pages = _split_pages(raw_txt)

    # frist_page
    first_page = pages[0]

    # clean
    first_page = _ultimate_clean(first_page)

    # id suposed begining of  true text
    i = _detect_true_text_id(first_page, line_length_txt=line_length_txt, n_lines=n_lines)
    # cand header and txt
    cand_header, cand_page_1 = _do_split_header_page(first_page, i)

    # pages
    pages[0] = cand_page_1

    # dump doubles lines
    cand_header = _del_bouble_breaks_and_spaces(cand_header)
    pages = [_del_bouble_breaks_and_spaces(i) for i in pages]

    # ultimate  clean
    cand_header = _ultimate_clean(cand_header)
    pages = [_del_dummy_breaklines(i) for i in pages]
    pages = [_ultimate_clean(i) for i in pages]

    # transfert last line if needed
    cand_header, pages[0] = _transfert_title_from_head_to_page(cand_header, pages[0])

    # detect section
    pages = [_detect_chapter(i) for i in pages]

    dd = {
        "header": cand_header,
        "pages": pages,
    }

    return dd


if __name__ == "__main__":

    # import
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y

    #  dataframe and structured_legal_doc_
    df = legal_doc_X_y(features="defendant")
    df["structured_txt"] = [
        structure_legal_doc(i, line_length_txt=50, n_lines=5) for i in df.txt.values
    ]

    df["header"] = df.structured_txt.apply(lambda i: i["header"])
    df["first_page"] = df.structured_txt.apply(lambda i: i["pages"][0])

    keys = ["folder", "header", "first_page"]
    result = df.loc[:, keys]
    result.to_csv("./data/csv/alex_work_21-06-16.csv", index=False)



import os

from legal_doc_processing import logger

from legal_doc_processing.press_release.utils import product_juridic_form


def _clean_LLC_trailling_dot_comma(txt: str) -> str:
    """ del LLC, Llc, LTD. etc etc"""

    # llc
    list_jur_form = product_juridic_form()
    for j in list_jur_form:
        txt = txt.replace(j, "")

    # trailling . or,
    if len(txt):
        txt = txt[:-1] if txt[-1] in [",", "."] else txt
        txt = txt[:-1] if txt[-1] in [",", "."] else txt

    return txt


def _clean_and(ans_list: list) -> list:
    """we want ['alex', 'cecile', 'alex and cecile' ]  became ['alex', 'cecile'] """

    l = list()
    for ans in ans_list:
        if " and " not in ans:
            l.append(ans)
        else:
            ll = ans.split(" and ")
            ll = [i.strip() for i in ll]
            ll = list(set(ll))
            l.extend(ll)

    l = list(set(l))
    return l


def _clean_resident(txt: str) -> str:
    """delete resident """

    l = txt.split(" ")
    resident = [i for i, j in enumerate(l) if j.startswith("Resident")]
    # if not resident
    if len(resident) != 1:
        return txt
    # else
    l = l[resident[0] + 1 :]

    return " ".join(l)


def _clean_defendants(ans_list: list) -> list:
    """delete defenants """

    ans_list = [i for i in ans_list if (i.lower() != "defendants")]
    ans_list = [i for i in ans_list if (i.lower() != "defendant")]

    del_defendants = lambda i, defendant: i.strip().replace(defendant, "").strip()

    defendant_list = ["Defendants", "Defendant", "defendant", "defendants"]

    for d in defendant_list:
        ans_list = [del_defendants(i, d) for i in ans_list]

    return ans_list


def _sub_you_shall_not_pass(
    ans_list,
    defendants=True,
    too_long=True,
    too_short=True,
    _lower=True,
    _and=True,
    llc=True,
    resident=True,
):
    """welcome in the rocky horror picture function
    here we are gonna take an list of text and make a bunch of horrible but
    working cleaning operation.
    the goal is to make each answer more -generic- to make merge more easy and to avoid a list of
    very close but slicely  different answer
    example ["alex", "Alex", "Alex LLC", "Alex LLC and paul"] become ["alex", "paul"]"""

    # strip
    ans_list = [i.strip() for i in ans_list]

    # clean defendants
    if defendants:
        ans_list = _clean_defendants(ans_list)

    # dummy words
    forbiden = [
        "Judge",
        "Civil Monetary Penalty",
        "Anti-Money Laundering",
        "Swap Reporting violations",
        "Commodity Industry",
        "Personal Expenses",
        "Commodity Futures",
        "Commodity Exchange ",
        "Federal Court",
        "CFTC",
        "U.S. District Court",
        "Commodity Exchange",
        "an Unregistered",
        "Swap Dealer",
        "Commodity Trading",
        "Commodity Pool",
        "Court",
        "Federal",
        "Consumer Financial Protection Bureau",
        "CFPB",
        "Respondents",
        "respondents",
        "SEC",
    ]

    for f in forbiden:
        ans_list = [i for i in ans_list if (f.lower() not in i.lower())]

    # duplicate
    ans_list = set(ans_list)

    # len to long
    if too_long:
        ans_list = [i for i in ans_list if len(i) < 60]

    if too_short:
        ans_list = [i for i in ans_list if len(i) > 3]

    # all lower
    if _lower:
        ans_list = [i for i in ans_list if i.lower() != i]

    # and
    if _and:
        ans_list = _clean_and(ans_list)

    # LLC
    if llc:
        ans_list = [_clean_LLC_trailling_dot_comma(i) for i in ans_list]

    # clean resident
    if resident:
        ans_list = [_clean_resident(i) for i in ans_list]

    ans_list = set(ans_list)

    return ans_list


def _you_shall_not_pass(ans_list: list) -> list:
    """make _sub_shall_not_pass twice due to the -and- problem :
    ["alex and paul",] should become become ["alex", paul"]
    if so we need to clean twice"""

    # fiest clean
    _ans_list = _sub_you_shall_not_pass(ans_list)

    # sep with comma
    new_list = list()
    for ans in _ans_list:
        sub_l = ans.split(",")
        new_list.extend(sub_l)

    # reclean
    _new_list = _sub_you_shall_not_pass(new_list)

    return _new_list


def clean_ans(ans: list) -> list:
    """ans is a list of dict. each dict is  : {answer:"foo", score:0.32}.
    for each dict,  add and _id and a new_ans based on the _you_shall_not_pass method
    the _you_shall_not_pass method is able to ditect:
     - completly inconsistant answer, if so the answer is droped
     - not so consistant answer, or non uniformized answer, if so the new_answer is the -more generic-
     version of ansxer "
     last but not least, and answer could be 'foo and bar' but this is indeed 2 answers
     'foo' and 'bar'. In this case we will create from one dict 2 dicts with same properties but separate
     new_ans
     before ans is a list of one dict -> [{answer:"foo and bar", score :0.123456},]
     after ans is a list of 2 dicts ->   [{new_answer:'foo', answer:"foo and bar", score :0.123456},
                                         {new_answer:'bar', answer:"foo and bar", score :0.123456}]"""

    # ans = copy.deepcopy(ans)

    # clean ans
    _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
    _ = [d.update({"new_answer": _you_shall_not_pass([d["answer"]])}) for d in ans]

    new_ans = list()
    for i, d in enumerate(ans):
        if len(d["new_answer"]) == 0:
            # ans.pop(i)
            pass
        if len(d["new_answer"]) == 1:
            # d["new_answer"] = list(d["new_answer"])[0]
            new_ans.append(
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": list(d["new_answer"])[0],
                }
            )
            # ans.pop(i)
        if len(d["new_answer"]) > 1:
            l = [
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": k,
                }
                for k in d["new_answer"]
            ]
            new_ans.extend(l)
            # ans.pop(i)

    return new_ans



import pandas as pd

from legal_doc_processing import logger

from legal_doc_processing.utils import (
    _if_not_spacy,
    _if_not_pipe,
    _ask,
    # get_pers,
    # get_orgs,
    get_label_,
    main_X_y,
)


def decision_X_y(juridiction="", features="", sample=1.0):
    """ """

    # main
    main_df = main_X_y()

    # press_release_text not na
    at_least_one_na = main_df.press_release_text.isna() + main_df.legal_doc_text.isna()
    main_df = main_df.loc[~at_least_one_na, :]

    # juridiction
    filter_jur = lambda i: juridiction.strip().lower() == str(i).strip().lower()
    jur_df = (
        main_df.loc[main_df["juridiction"].apply(filter_jur), :]
        if juridiction
        else main_df
    )

    # sample
    jur_df = jur_df.sample(frac=sample).reset_index(drop=True)

    # features
    if not features:
        drop_cols = [i for i in jur_df if ("press_" in i)]
        jur_df.drop(drop_cols, axis=1, inplace=True)
        return jur_df

    if isinstance(features, str):
        features = [features]

    keep_cols = [
        "folder",
        "press_release_link",
        "press_release_link_new",
        "press_release_text",
        "legal_doc_text",
        "document_link",
        "document_link_new",
        "legal_doc_text",
    ] + features

    features_df = jur_df.loc[:, keep_cols]

    return features_df



from legal_doc_processing import logger


def _clean_str_to_str(txt: str) -> str:
    """ """

    return str


def _clean_list_to_list(ans_list: list, clean_str=_clean_str_to_str) -> list:
    """ """

    ans_list = [clean_str(i) for i in ans_list if i.strip()]

    return ans_list


def clean_ans(ans, clean_str=_clean_str_to_str, clean_list=_clean_list_to_list):
    """ """

    # ans = copy.deepcopy(ans)

    # clean ans
    _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
    _ = [d.update({"new_answer": clean_list([d["answer"]], clean_str)}) for d in ans]

    new_ans = list()
    for i, d in enumerate(ans):
        if len(d["new_answer"]) == 0:
            # ans.pop(i)
            pass
        if len(d["new_answer"]) == 1:
            # d["new_answer"] = list(d["new_answer"])[0]
            new_ans.append(
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": list(d["new_answer"])[0],
                }
            )
            # ans.pop(i)
        if len(d["new_answer"]) > 1:
            l = [
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": k,
                }
                for k in d["new_answer"]
            ]
            new_ans.extend(l)
            # ans.pop(i)

    return new_ans



from legal_doc_processing.press_release.information_extraction.defendant import (
    _clean_LLC_trailling_dot_comma,
    _clean_and,
    _clean_resident,
    _clean_defendants,
    _you_shall_not_pass,
)


class Test_clean_LLC_trailling_dot_comma(unittest.TestCase):
    """Test Class for LLC """

    def test_ok(self):
        """test succed """

        cands = [
            ("A Gaz LLC", "A Gaz"),
            ("A Gaz, LLC.", "A Gaz"),
            ("A Gaz, LLC", "A Gaz"),
        ]
        X, y = zip(*cands)

        preds = [_clean_LLC_trailling_dot_comma(x) for x in X]

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9


class Test_clean_and(unittest.TestCase):
    """Test Class and"""

    def test_ok(self):
        """test succeed """

        cands = [
            (["Alex and Lucy"], ["Alex", "Lucy"]),
        ]
        X, y = zip(*cands)

        preds = [_clean_and(x) for x in X]

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9


class Test_clean_and(unittest.TestCase):
    """Test Class and"""

    def test_ok(self):
        """test succeed """

        cands = [
            (["Alex and Lucy"], ["Alex", "Lucy"]),
        ]
        X, y = zip(*cands)

        preds = [_clean_and(x) for x in X]

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9


class Test_clean_resident(unittest.TestCase):
    """Test Class _clean_resident"""

    def test_ok(self):
        """test succeed """

        cands = [
            ("Alex Gazagnes", "Alex Gazagnes"),
            ("Rouen Resident Alex Gazagnes", "Alex Gazagnes"),
            ("former Resident Alex Gazagnes", "Alex Gazagnes"),
        ]

        X, y = zip(*cands)

        preds = [_clean_resident(x) for x in X]

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9


class Test_clean_defendants(unittest.TestCase):
    """ """

    def test_ok(self):
        """ """

        cands = ans_list = [
            ("defendant Alex Gazagnes", "Alex Gazagnes"),
            ("Defendants Alex Gazagnes", "Alex Gazagnes"),
        ]

        X, y = zip(*cands)

        preds = _clean_defendants(X)

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9


class Test_you_shall_not_pass(unittest.TestCase):
    """Test Class_you_shall_not_pass"""

    def test_ok(self):
        """test succeed """

        ans_list = X = [
            "defendant",
            "defendants",
            "ejidej deznuindniinin nuin",
            "hduehizhdiheihifzeifhihihefhifuhiehueih uheiz hfuihfuize huizeh uifhuifhzeuhf uizhfuieh zehfzu hizehu eizhf izuh",
            "hduehizhdiheihifzeifhihihefhifuhiehueih uhe, hfuihfuize huizeh uifhuifhzeuhf uizhfuieh zehfzu hizehu eizhf izuh",
            "Alex Gazagnes LLC",
            "Alex Gazagnes LLC",
            "Alex Gazagnes, Ltd.",
            "   Rouen Resident Alex Gazagnes   ",
            "former Resident Alex Gazagnes",
            "defendant Alex Gazagnes",
            "Alex Gazagnes and John Doe",
            "AlexI Gazagnes, JohnI Doe LLC.",
            "AlexA Gazagnes, JohnA Doe LLC.",
            "Alex Gazagnes, LLC. and John Doe LLC.",
            "AlexI Gazagnes, LLC. and JohnI Doe LLC.",
            "societe international LLC, Alex Gazagnes, John Doe and Fran Fine",
        ]

        y = [
            "Alex Gazagnes",
            "John Doe",
            "AlexI Gazagnes",
            "JohnI Doe",
            "AlexA Gazagnes",
            "JohnA Doe",
            "societe international,  Fran Fine",
        ]

        preds = [_you_shall_not_pass(x) for x in X]

        res = list(zip(X, y, preds))
        eqs = [(i == j) for i, j in zip(y, preds)]
        acc = sum(eqs) / len(eqs)
        assert acc > 0.9



import os
from pprint import pformat, pprint
import random

from legal_doc_processing import logger

from legal_doc_processing.utils import get_spacy, get_pipeline, get_label_

from legal_doc_processing.press_release.utils import press_release_X_y

from legal_doc_processing.press_release.structure.utils import (
    clean_in_line_break,
    do_strip,
    clean_very_short_lines,
)

from legal_doc_processing.press_release.structure.sec_model_1 import (
    split_intro_article_1,
    extract_id_1,
    extract_h1_1,
    extract_date_1,
)


from legal_doc_processing.press_release.structure.sec_model_2 import (
    split_intro_article_2,
    extract_id_2,
    extract_h1_2,
    extract_date_2,
)


def give_sec_press_release_df():
    """ """

    df = press_release_X_y(juridiction="sec", sample=0.9)
    cols = ["folder", "press_release_text"]

    return df.loc[:, cols]


def give_sec_press_release_file(model=1):
    """ """

    if model == 1:

        folder_list = [
            "2020-177",
            "2017-133",
            "2016-64",
            "2019-260",
            "2018-212",
            "2011-146",
            "2015-170",
            "2019-102",
            "2016-224",
            "2017-18",
            "2015-165",
            "2019-260",
            "2019-260",
            "2020-254",
            "2019-12",
            "2011-87",
            "2012-266",
            "2011-87",
            "2017-18",
            "2015-170",
        ]

    else:
        folder_list = [
            "lr17887",
            "lr21920",
            "lr20029",
            "lr21156",
            "lr20319",
            "lr21920",
            "lr21617",
            "lr18081",
            "lr20029",
            "lr21617",
            "lr21920",
            "lr20835",
            "lr17887",
            "lr20319",
            "lr21229",
        ]

    folder = random.choice(folder_list)
    fn = f"./data/files/sec/{folder}/press-release.txt"

    with open(fn, "r") as f:
        txt = f.read()

    return txt


def first_clean(txt: str) -> str:
    """ """

    btxt = txt.encode("latin-1")
    txt_decoded = btxt.decode("utf8")

    # # clean double breaks and fake lines
    # new_txt_1 = clean_in_line_break(txt)

    # strip
    new_txt_2 = do_strip(txt_decoded)

    return new_txt_2


def define_sec_press_release_model(txt):
    """ """

    if "FOR IMMEDIATE RELEASE".lower() in txt[:1000].lower():
        return 1
    else:
        return 2


def structure_press_release(txt, nlspa=""):
    """ """

    # spacy
    if not nlspa:
        nlspa = get_spacy()
    try:
        nlspa.add_pipe("sentencizer")
    except Exception as e:
        pass

    dd = {
        "folder": "--ERROR--",
        "date": "--ERROR--",
        "h1": "--ERROR--",
        "article": "--ERROR--",
        "end": "--ERROR--",
        "error": "0",
    }

    sec_model = define_sec_press_release_model(txt)

    if sec_model == 2:
        try:

            # clean
            cleaned_txt = first_clean(txt)

            # intro article
            intro, article = split_intro_article_2(cleaned_txt)

            # date
            dd["date"], intro_1 = extract_date_2(intro, nlspa)

            # h1
            dd["h1"], _ = extract_h1_2(intro_1)

            # clean article
            cleaned_article = clean_in_line_break(article)
            # dd["error"] = "last line ok 117 "

            # article
            dd["article"] = cleaned_article
            # dd["error"] = "last line ok 121 "

            dd["error"] = "0"

        except Exception as e:
            logger.error(e)
            dd["error"] += str(e)

    if sec_model == 1:
        try:
            # clean
            cleaned_txt = first_clean(txt)
            # dd["error"] = "last line ok 96 "

            # intro article
            intro, article = split_intro_article_1(cleaned_txt)
            # dd["error"] = "last line ok 100 "

            # id
            dd["folder"], intro_2 = extract_id_1(intro)
            # dd["error"] = "last line ok 104 "

            # h1
            dd["h1"], _ = extract_h1_1(intro_2)
            # dd["error"] = "last line ok 108 "

            # h1
            dd["date"], _ = extract_date_1(article, nlspa)
            # dd["error"] = "last line ok 113 "

            # clean article
            cleaned_article = clean_in_line_break(article)
            # dd["error"] = "last line ok 117 "

            # article
            dd["article"] = cleaned_article
            # dd["error"] = "last line ok 121 "

            dd["error"] = "0"

        except Exception as e:
            logger.error(e)
            dd["error"] += str(e)

    return dd



from legal_doc_processing import logger


from collections import Counter
from legal_doc_processing.utils import softmax, merge_ans


def predict_defendant(
    press: dict,
    legal: dict,
    threshold: float = 0.1,
    softmax_threshold: float = 0.2,
) -> list:
    """ """

    press_def = press["_defendant"]
    legal_def = legal["_defendant"]

    ans = []

    print(press_def)
    print(legal_def)

    # both null
    if (not press_def) and not (legal_def):
        ans = [(-1, -1)]
    # one null
    elif not press_def:
        ans = legal_def
    elif not (legal_def):
        ans = press_def

    if ans:
        print(str(ans) + "\n")
        return ans

    # join preds
    all_def = press_def + legal_def

    # non null
    selected_def = [(i, j) for i, j in all_def if j > 0.1]
    if not selected_def:
        ans = [(-2, -1)]

    if ans:
        print(str(ans) + "\n")
        return ans

    # counter
    counter = Counter([i for i, _ in selected_def])
    twos = [i for i, j in counter.items() if j > 1]

    # both the same
    if len(press_def) == len(legal_def):
        if len(twos) == len(press_def):
            both = [{"answer": i, "score": j} for i, j in selected_def]
            merged_both = merge_ans(both, "answer")
            ans = [(i["answer"], i["cum_score"]) for i in merged_both]

    if ans:
        print(str(ans) + "\n")
        return ans

    # softmax
    keys, values = zip(*press_def)
    _values = [round(i, 4) for i in softmax(values)]
    softmax_press = zip(keys, _values)
    keys, values = zip(*legal_def)
    _values = [round(i, 4) for i in softmax(values)]
    softmax_legal = zip(keys, _values)

    # over weight ans in both
    update_tuple = lambda i, j: (i, j) if (i not in twos) else (i, j + 0.3)
    new_press = [update_tuple(i, j) for i, j in softmax_press]
    new_legal = [update_tuple(i, j) for i, j in softmax_legal]

    # new_selected_df as dict
    new_selected_df = new_legal + new_press
    new_selected_df = [{"answer": i, "score": j} for i, j in new_selected_df]
    merged_new = merge_ans(new_selected_df, "answer")
    ans = [(i["answer"], i["cum_score"]) for i in merged_new]

    # softmax threshold if len 4
    if len(ans) >= 4:
        keys, values = zip(*ans)
        _values = [round(i, 4) for i in softmax(values)]
        ans = zip(keys, _values)
        ans = [(i, j) for i, j in ans if j > softmax_threshold]

    print(str(ans) + "\n")
    return ans



from legal_doc_processing import logger
from legal_doc_processing._legal_doc import LegalDoc
from legal_doc_processing._press_release import PressRelease
from legal_doc_processing.base import (
    Base,
)


class Decision(Base):
    def __init__(
        self,
        press_release_text: str,
        legal_doc_text: str,
        source,
        nlpipe=None,
        nlspa=None,
    ):

        Base.__init__(
            self,
            text="",
            obj_name="Decision",
            source=source,
            nlpipe=nlpipe,
            nlspa=nlspa,
        )

        self.legal_doc = LegalDoc(
            legal_doc_text,
            source=source,
            nlpipe=nlpipe,
            nlspa=nlspa,
        )
        self.press_release = PressRelease(
            press_release_text,
            source=source,
            nlpipe=nlpipe,
            nlspa=nlspa,
        )

    ######################

    def predict(self, feature) -> str:
        """ """

        if feature == "all":
            return self.predict_all()

        if feature in self.feature_list:
            # try:
            self.press_release.predict(feature)
            self.legal_doc.predict(feature)
            val = self._predict[feature](
                self.press_release._feature_dict, self.legal_doc._feature_dict
            )
            setattr(self, "_" + feature, val)
            return val
            # except:
            #     return "--Error--"
        return "--Unknowned feature--"

    def predict_all(self) -> str:
        """return self.predict("all") """

        for feature in self.feature_list:

            self.press_release.predict_all()
            self.legal_doc.predict_all()

            setattr(
                self,
                "_" + feature,
                self._predict[feature](self.press_release.data, self.legal_doc.data),
            )


class _Decision:
    Decision = Decision


# def from_file(press_release, legal_doc, nlpipe=None, nlspa=None):

#     nlpipe = nlpipe if nlpipe else get_pipeline()
#     nlspa = nlspa if nlspa else get_spacy()
#     try:
#         nlspa.add_pipe("sentencizer")
#     except Exception as e:
#         pass

#     press_release_text = ""
#     try:
#         with open(press_release, "r") as f:
#             press_release_text = f.read()
#     except Exception as e:
#         print(e.__str__())

#     legal_doc_text = ""
#     try:
#         with open(legal_doc, "r") as f:
#             legal_doc_text = f.read()
#     except Exception as e:
#         print(e.__str__())

#     if legal_doc_text and press_release_text:
#         return Decision(
#             press_release_text,
#             legal_doc_text,
#             press_release_file_path=press_release,
#             legal_doc_file_path=legal_doc,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     elif legal_doc_text and (not press_release_text):
#         return LegalDoc(
#             legal_doc_text,
#             file_path=legal_doc,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     elif (not legal_doc_text) and press_release_text:
#         return PressRelease(
#             press_release_text,
#             file_path=press_release,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     else:
#         raise NotImplementedError("something went wrong mother fucker :) ")


# def from_text(press_release, legal_doc, nlpipe=None, nlspa=None):

#     nlpipe = nlpipe if nlpipe else get_pipeline()
#     nlspa = nlspa if nlspa else get_spacy()
#     try:
#         nlspa.add_pipe("sentencizer")
#     except Exception as e:
#         pass

#     if legal_doc and press_release:
#         return Decision(
#             press_release,
#             legal_doc,
#             press_release_file_path=press_release,
#             legal_doc_file_path=legal_doc,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     elif legal_doc and (not press_release):
#         return LegalDoc(
#             legal_doc,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     elif (not legal_doc) and press_release:
#         return PressRelease(
#             press_release,
#             nlpipe=nlpipe,
#             nlspa=nlspa,
#         )
#     else:
#         raise NotImplementedError("something went wrong mother fucker :) ")


# if __name__ == "__main__":

#     # import
#     import time
#     from legal_doc_processing.utils import get_pipeline, get_spacy
#     from legal_doc_processing.decision.utils import decision_X_y

#     # load
#     nlpipe = get_pipeline()
#     nlspa = get_spacy()
#     nlspa.add_pipe("sentencizer")

#     # legal_doc df AND  OBj
#     df = decision_X_y()
#     df = df.iloc[:, :]
#     # df["obj"] = df.txt.apply(lambda i: LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa))

#     # clean
#     df["press_txt"] = df.press_txt.fillna("")
#     df["legal_txt"] = df.legal_txt.fillna("")

#     _dec = lambda i, j: Decision(i, j, nlpipe=nlpipe, nlspa=nlspa)
#     df["ds"] = [_dec(i, j) for i, j in zip(df.press_txt.values, df.legal_txt.values)]

#     # preds
#     t = time.time()
#     # 28 objects --> 181 secondes so --> +/-10 secondes per objects
#     df["pred_defendant"] = df.ds.apply(lambda i: i.predict("defendant"))
#     t = time.time() - t

#     #     # labels
#     #     preds_labels = list(df.preds.iloc[0].keys())
#     #     for k in preds_labels:
#     #         df["pred_" + k] = df.preds.apply(lambda i: i[k])

#     # 1st one
#     one = df.iloc[0, :]
#     one_ob = obj = self = one.ds


# import
import os
from collections import Counter

import requests

import pandas as pd

from legal_doc_processing.downloader.google import files
from legal_doc_processing.downloader.filters import (
    filter_juridiction,
    filter_ext,
    filter_press_release,
    select_best_file,
    handle_multiple_file_problem,
)
from legal_doc_processing.downloader.clean import clean_url
from legal_doc_processing.downloader.get import get_text
from legal_doc_processing.downloader.save import save_files


SOURCE = "cftc"


def first_example():
    """ """

    url = "https://storage.googleapis.com/theolex_documents_processing/cftc/text/7100-15/press-release.txt"
    req = requests.get(url)

    return req.text


def download_text_from_bucket(SOURCE, STOP=10):
    """ """

    # check args
    assert SOURCE in ["doj", "sec", "cftc", "cfbp"]

    # filters
    filter__txt = lambda i: filter_juridiction(filter_ext(i, "txt"), SOURCE)
    _txt = [i for i in files if filter__txt(i)]
    print(str(_txt[:3]) + "\n")
    # print(str(sec_txt[:3]) + "\n")

    # sep press release
    doc_list_ = [i for i in _txt if filter_press_release(i, False)]
    press_list_ = [i for i in _txt if filter_press_release(i, True)]
    print(doc_list_[:3])
    print(press_list_[:3])

    # pairs folder / url
    make_dict = lambda i: [i.split("/")[-2], i]
    #
    doc_pairs_ = [(i, j) for i, j in map(make_dict, doc_list_)]
    press_pairs_ = [(i, j) for i, j in map(make_dict, press_list_)]
    # check
    print(doc_pairs_[:5])
    print(press_pairs_[:5])

    # we have multiple 'legal doc' files for each folder complaint-allied-markets-llc-et-al.txt' and order-allied-markets-llc-et-al.txt
    # proof count folder
    # counts_before = Counter([i for i, j in doc_pairs_])
    # counts_before = [(i, j) for i, j in counts_before.items() if j > 1]
    # counts_before[:10]
    # do clean
    doc_pairs__cleaned = handle_multiple_file_problem(doc_pairs_)
    # proof count folder -> only unique
    # counts_after = Counter([i for i, j in doc_pairs__cleaned])
    # counts_after = [(i, j) for i, j in counts_after.items() if j > 1]
    # counts_after[:5]

    # make a dict of each
    doc_dict_ = {i: j for i, j in doc_pairs__cleaned}
    press_dict_ = {i: j for i, j in press_pairs_}

    print([(i, j) for i, j in doc_dict_.items()][:3])
    print([(i, j) for i, j in press_dict_.items()][:3])

    # build a df
    doc_df_dict_ = pd.DataFrame(
        [{"folder": i, "document_link": j} for i, j in doc_pairs__cleaned]
    )
    press_df_dict_ = pd.DataFrame(
        [{"folder": i, "press_release_link": j} for i, j in press_pairs_]
    )
    original_df = pd.merge(doc_df_dict_, press_df_dict_, how="outer", on="folder")
    sorted_df = original_df.sort_values("folder", axis=0, inplace=False)
    sorted_df.head(5)

    # clean url to have ok urls
    sorted_df["document_link_new"] = sorted_df.document_link.apply(clean_url)
    sorted_df["press_release_link_new"] = sorted_df.press_release_link.apply(clean_url)
    # check
    print(sorted_df.shape)
    print(sorted_df["document_link_new"].iloc[:5].values)
    print(sorted_df["press_release_link_new"].iloc[:5].values)
    sorted_df.head(5).T

    # keep ok url press link OR ok url doc link
    _idx = sorted_df.document_link_new.apply(
        bool
    ) + sorted_df.press_release_link_new.apply(bool)
    final_df = sorted_df.loc[_idx, :]
    final_df.head(5)

    # shuffle and reindex
    sample_df = final_df.sample(frac=1).reset_index(drop=True)

    # SOURCE
    sample_df["juridiction"] = SOURCE

    # SELECT
    if STOP > 0:
        sample_df = sample_df.iloc[:STOP, :]
    print(sample_df.shape)
    sample_df = sample_df.sort_values("folder", axis=0, inplace=False)

    # do download
    sample_df["legal_doc_text"] = sample_df.document_link_new.apply(get_text)
    sample_df["press_release_text"] = sample_df.press_release_link_new.apply(get_text)

    # # save
    # tmp_filename = f"{SOURCE}/data/csv/o00_{SOURCE}_text_from_bucket.csv"
    # sample_df.to_csv(tmp_filename, index=False)
    # sample_df = pd.read_csv(tmp_filename)

    # # save in files
    # sample_df["document_link_new"] = sample_df.document_link_new.fillna("")
    # sample_df["press_release_link_new"] = sample_df.press_release_link_new.fillna("")
    # save_files(sample_df, path=f"{SOURCE}/data/files/")


class Downloader:
    first_example = first_example


import os
from pprint import pformat, pprint

from legal_doc_processing import logger

from legal_doc_processing.utils import get_spacy, get_pipeline, get_label_

from legal_doc_processing.press_release.utils import press_release_X_y

from legal_doc_processing.press_release.structure.utils import (
    clean_in_line_break,
    do_strip,
    clean_very_short_lines,
)


def give_cftc_press_release_df():
    """ """

    df = press_release_X_y(juridiction="cftc", sample=0.9)
    cols = ["folder", "press_release_text"]

    return df.loc[:, cols]


def give_cftc_press_release_file():
    """ """

    fn = "./data/files/cftc/7117-15/press-release.txt"

    with open(fn, "r") as f:
        txt = f.read()

    # btxt = txt.encode("latin-1")
    # txt_decoded = btxt.decode("utf8")

    return txt


def first_clean(txt: str) -> str:
    """ """

    btxt = txt.encode("latin-1")
    txt_decoded = btxt.decode("utf8")

    # # clean double breaks and fake lines
    # new_txt_1 = clean_in_line_break(txt)

    # strip
    new_txt_2 = do_strip(txt_decoded)

    return new_txt_2


def split_intro_article(txt: str, n=30) -> str:
    """ """

    txt_lines_30 = txt.splitlines()[:n]

    s0, s1, s2 = ("Washington D", "Washington, D", "Washington,D")
    is_in = lambda j: any([(j.startswith(s.lower())) for s in [s0, s1, s2]])
    idx_cands = [i for i, j in enumerate(txt_lines_30) if is_in(j.lower())]

    if not len(idx_cands):
        s = "washington"
        idx_cands = [i for i, j in enumerate(txt_lines_30) if j.lower().startswith(s)]

    idx = idx_cands[0]

    txt_lines = txt.splitlines()
    intro_lines, article_lines = txt_lines[:idx], txt_lines[idx:]

    intro, article = "\n".join(intro_lines), "\n".join(article_lines)

    return intro, article


def extract_date(intro: str, nlspa) -> tuple:
    """ """

    date_list = get_label_(intro, label="DATE", nlspa=nlspa)

    # if date_list
    if len(date_list) > 0:
        date = date_list[0]
    else:
        date = "-1"

    # if not return dummy
    if date == "-1":
        return date, intro

    # find date_line
    intro_lines = intro.splitlines()
    idx_list = [i for i, j in enumerate(intro_lines) if date.lower() in j.lower()]
    idx = idx_list[0]

    # clean intro
    intro_lines[idx] = ""
    intro_ok = "\n".join(intro_lines)

    return date, intro_ok


def extract_id(intro: str, len_max=35) -> tuple:
    """ """

    # find id list
    # intro_lines = [i for i in intro.splitlines() if len(i) < len_max]
    intro_lines = [i for i in intro.splitlines()]

    idx_list = [
        i for i, j in enumerate(intro_lines) if j.lower().startswith("Release".lower())
    ]

    # return dummy
    if len(idx_list) != 1:
        return "-1", intro

    # clean intro
    idx = idx_list[0]
    _id = intro_lines[idx]
    intro_lines[idx] = ""
    intro_ok = "\n".join(intro_lines)

    return _id, intro_ok


def extract_h1(intro: str) -> tuple:
    """ """

    light_intro = clean_very_short_lines(intro)
    light_intro.replace("\n\n", "\n").replace("\n\n", "\n")
    intro_lines = light_intro.splitlines()
    h1 = ". ".join(intro_lines).strip()
    h1 = h1 if h1[-1] == "." else h1 + "."

    return h1, "-1"


def extract_end(article: str, n: int = -2) -> tuple:
    """ """

    split_article = article.split("\n\n")
    article_ok = "\n\n".join(split_article[:n])
    end = "\n\n".join(split_article[n:])

    return end, article_ok


def structure_press_release(txt, nlspa=""):
    """ """

    # spacy
    if not nlspa:
        nlspa = get_spacy()
    try:
        nlspa.add_pipe("sentencizer")
    except Exception as e:
        pass

    dd = {
        "folder": "--ERROR--",
        "date": "--ERROR--",
        "h1": "--ERROR--",
        "article": "--ERROR--",
        "end": "--ERROR--",
        "error": "0",
    }

    try:

        # clean
        cleaned_txt = first_clean(txt)

        # intro article
        intro, article = split_intro_article(cleaned_txt)

        # date
        dd["date"], intro_1 = extract_date(intro, nlspa)

        # id
        dd["folder"], intro_2 = extract_id(intro_1)

        # h1
        dd["h1"], _ = extract_h1(intro_2)

        # clean article
        cleaned_article = clean_in_line_break(article)

        # end
        dd["end"], article_ok = extract_end(cleaned_article)

        # article
        dd["article"] = article_ok

    except Exception as e:
        logger.error(e)
        dd["error"] = str(e)

    return dd



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all, get_label_
from legal_doc_processing.press_release.judge.questions import (
    _question_helper,
    _question_selector,
    _question_lister,
)
from legal_doc_processing.press_release.judge.clean import clean_ans_list, clean_ans


def predict_judge(data: dict, threshold: float = 0.75) -> list:
    """ """

    # make sent list, and filter not judge in sent
    sent_list = data.content_sents
    filter_judge = lambda j: ("judge" or "attorney") in j.lower()
    judge_sent_list = [(i, j) for i, j in enumerate(sent_list) if filter_judge(j)]

    # if no sents :
    if not judge_sent_list:
        return [("", 1)]

    # quest
    ans_list = list()
    for i, sent in judge_sent_list:
        key_list = _question_helper(sent)
        if key_list:
            quest_pairs = _question_lister(key_list)
            ans_list.extend(
                ask_all(sent, quest_pairs, sent_id=i, sent=sent, nlpipe=data.nlpipe)
            )

    # logger.info(f"ans_list : {ans_list}")

    if not len(ans_list):
        return [("", 1)]

    # clean ans
    cleaned_ans = clean_ans_list(ans_list)
    answer_label = "new_answer"

    # merge ans
    merged_ans = merge_ans(cleaned_ans, label=answer_label)

    # filter person list
    # pers_list = _u([clean_ans(i) for i in data.pers_all])  # unique
    # is_in = lambda i: i[answer_label] in pers_list
    # filtered_ans = [i for i in merged_ans if is_in(i)]

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in merged_ans]
    final_ans = [(i, j) for i, j in flatten_ans if j >= threshold]

    # if no ans :
    if not final_ans:
        return [("", 1)]

    return [final_ans[0]]



import os
import pdb
from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u
from legal_doc_processing.utils import get_label_

from legal_doc_processing.press_release.extracted_authorities.juridiction_pairs import (
    juridiction_dict,
)


def predict_extracted_authorities(data: dict) -> list:
    """ """

    # juridiction
    juridiction = [data.juridiction.upper()]

    # logger.info(f"juridiction : {juridiction} ")

    # courts
    courts = data._feature_dict["_court"]
    if courts:
        courts = [i for i, j in courts]
    else:
        courts = []

    # orgs
    all_orgs = list()
    for sent in data.content_sents:
        all_orgs.extend(get_label_(sent, "ORG", nlspa=data.nlspa))

    # filter orgs by auth
    jur_get = lambda i: juridiction_dict.get(i.lower().strip(), "")
    filtered_orgs = [jur_get(i) for i in all_orgs]
    filtered_orgs = [i.lower().strip() for i in filtered_orgs if i]

    # auths list
    auths_list = [i for i in _u(juridiction + courts + filtered_orgs) if i]

    capitalizer = (
        lambda i: i.replace("cfpb", "CFPB")
        .replace("cfbp", "CFBP")
        .replace("sec", "SEC")
        .replace("doj", "DOJ")
        .replace("cftc", "CFTC")
    )
    return [(capitalizer(i), 1) for i in auths_list]



# from legal_doc_processing import logger
# from legal_doc_processing.legal_doc.legal_doc import LegalDoc
# from legal_doc_processing.legal_doc.legal_doc import from_file, from_text
# from legal_doc_processing.legal_doc.utils import legal_doc_X_y as load_X_y



from legal_doc_processing import logger


def predict_justice_type(data: dict) -> list:
    """ """

    # juridiction = [data.juridiction]

    if ("cftc" or "cfbp" or "cfpb") in data.juridiction:
        return [("U.S. - Civil", 1)]

    if ("doj" or "sec") in data.juridiction:
        return [("U.S. - Penal", 1)]

    return [("-- DUMMY --", 1)]



import legal_doc_processing.press_release.information_extraction as press_inf_ext
import legal_doc_processing.legal_doc.information_extraction as legal_doc_inf_ext
import legal_doc_processing.decision.information_extraction as decision_inf_ext


def predict_handler(obj_name):
    """ container for predict function """

    if obj_name == "PressRelease":
        return {
            "compliance_obligations": press_inf_ext.predict_compliance_obligations,
            "cooperation_credit": press_inf_ext.predict_cooperation_credit,
            "court": press_inf_ext.predict_court,
            "country_of_violation": press_inf_ext.predict_country_of_violation,
            "currency": press_inf_ext.predict_currency,
            "decision_date": press_inf_ext.predict_decision_date,
            "defendant": press_inf_ext.predict_defendant,
            "extracted_authorities": press_inf_ext.predict_extracted_authorities,
            "extracted_sanctions": press_inf_ext.predict_extracted_sanctions,
            "extracted_violations": press_inf_ext.predict_extracted_violations,
            "folder": press_inf_ext.predict_folder,
            "judge": press_inf_ext.predict_judge,
            "justice_type": press_inf_ext.predict_justice_type,
            "monetary_sanction": press_inf_ext.predict_monetary_sanction,
            "monitor": press_inf_ext.predict_monitor,
            "nature_de_sanction": press_inf_ext.predict_nature_de_sanction,
            "nature_of_violations": press_inf_ext.predict_nature_of_violations,
            "penalty_details": press_inf_ext.predict_penalty_details,
            "reference": press_inf_ext.predict_reference,
            "type": press_inf_ext.predict_type,
            # "violation_date ": predict_violation_date,
        }
    if obj_name == "LegalDoc":
        return {
            "compliance_obligations": legal_doc_inf_ext.predict_compliance_obligations,
            "cooperation_credit": legal_doc_inf_ext.predict_cooperation_credit,
            "court": legal_doc_inf_ext.predict_court,
            "country_of_violation": legal_doc_inf_ext.predict_country_of_violation,
            "currency": legal_doc_inf_ext.predict_currency,
            "decision_date": legal_doc_inf_ext.predict_decision_date,
            "defendant": legal_doc_inf_ext.predict_defendant,
            "extracted_authorities": legal_doc_inf_ext.predict_extracted_authorities,
            "extracted_violations": legal_doc_inf_ext.predict_extracted_violations,
            "extracted_sanctions": legal_doc_inf_ext.predict_extracted_sanctions,
            "folder": legal_doc_inf_ext.predict_folder,
            "judge": legal_doc_inf_ext.predict_judge,
            "justice_type": legal_doc_inf_ext.predict_justice_type,
            "monetary_sanction": legal_doc_inf_ext.predict_monetary_sanction,
            "monitor": legal_doc_inf_ext.predict_monitor,
            "nature_de_sanction": legal_doc_inf_ext.predict_nature_de_sanction,
            "nature_of_violations": legal_doc_inf_ext.predict_nature_of_violations,
            "penalty_details": legal_doc_inf_ext.predict_penalty_details,
            "reference": legal_doc_inf_ext.predict_reference,
            "type": legal_doc_inf_ext.predict_type,
            # "violation_date ": predict_violation_date,
        }
    if obj_name == "Decision":
        return {
            "compliance_obligations": decision_inf_ext.predict_compliance_obligations,
            "cooperation_credit": decision_inf_ext.predict_cooperation_credit,
            "court": decision_inf_ext.predict_court,
            "country_of_violation": decision_inf_ext.predict_country_of_violation,
            "currency": decision_inf_ext.predict_currency,
            "decision_date": decision_inf_ext.predict_decision_date,
            "defendant": decision_inf_ext.predict_defendant,
            "extracted_authorities": decision_inf_ext.predict_extracted_authorities,
            "extracted_sanctions": decision_inf_ext.predict_extracted_sanctions,
            "extracted_violations": decision_inf_ext.predict_extracted_violations,
            "folder": decision_inf_ext.predict_folder,
            "judge": decision_inf_ext.predict_judge,
            "justice_type": decision_inf_ext.predict_justice_type,
            "monetary_sanction": decision_inf_ext.predict_monetary_sanction,
            "monitor": decision_inf_ext.predict_monitor,
            "nature_de_sanction": decision_inf_ext.predict_nature_de_sanction,
            "nature_of_violations": decision_inf_ext.predict_nature_of_violations,
            "penalty_details": decision_inf_ext.predict_penalty_details,
            "reference": decision_inf_ext.predict_reference,
            "type": decision_inf_ext.predict_type,
            # "violation_date ": predict_violation_date,
        }


import os
import copy

import pandas as pd

from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.press_release.monetary_sanction.clean import _cast_as_int


def predict_monetary_sanction(
    data: dict,
    # h1_len_threshold: int = 15,
    # content_n_sents_threshold: int = 5,
    # threshold: float = 0.25,
) -> list:

    # # items
    # h1, abstract = data.h1, data.abstract

    # get_label_ h1,predict
    money_h1 = _u(data.cost_h1)
    money_h1_clean = _u(_cast_as_int(money_h1))

    # cost in h1
    if len(money_h1_clean) == 1:
        return [(str(money_h1_clean[0]), 1)]
    if len(money_h1_clean) > 1:
        return [(str(-2), 1)]

    # get_label abstract,predict
    money_abstract = _u(data.cost_abstract)
    money_abstract_clean = _u(_cast_as_int(money_abstract))

    # cost in article
    if len(money_abstract_clean) == 1:
        return [(str(money_abstract_clean[0]), 1)]
    elif len(money_abstract_clean) > 1:
        return [(str(max(money_abstract_clean)), 1)]
    else:
        return [(str(-1), -1)]

    return [(str(-3), -1)]


# def predict_monetary_sanction(data.: dict) -> list:
#     """init a pipe if needed, then ask all questions and group all questions ans in a list sorted py accuracy """

#     # pipe, spa
#     nlpipe, nlspa = data.["nlpipe"], data.["nlspa"]



from legal_doc_processing import logger

from legal_doc_processing.press_release.utils import product_juridiction_pairs


def _filter_jur(token, cands: list = None):
    """ """

    if not cands:
        cands = product_juridiction_pairs()

    for k, v in cands.items():
        if token.lower().strip() == k.lower().strip():
            return v.upper()

    return ""



def final_clean(txt: str) -> str:
    """ """

    # US
    txt = txt.replace("United States", "U.S")

    # errors
    errors_list = [
        "court of law",
        "judgment and order",
        "court order",
        "court’s order",
        "court approval",
    ]
    if txt.lower().strip() in errors_list:
        return ""

    # mandatories
    mand_list = ["court", "district", "federal", "tribunal", "us", "u.s", "united states"]
    _txt = txt.lower()
    if not any([(i in _txt) for i in mand_list]):
        return ""

    # \n
    slash_n_clean = (
        lambda j: j.replace(".\n", ". \n")
        .replace("\n", " ")
        .replace("  ", " ")
        .replace("  ", " ")
        .strip()
    )

    return slash_n_clean(txt)


from legal_doc_processing.utils import uniquize as _u


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    cands = [
        "impose",
        "judgment",
        "order",
        "settl",
        "enters",
        "act",
    ]

    for cand in cands:
        if cand in _txt:
            res.append(cand)

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    res = list()

    if key in ["impose", "judgment", "order", "settl", "enters", "act"]:
        res.extend(
            [
                ("Who is the nature of the decision?", "nature"),
            ]
        )
    return res


def _question_lister(key_list: list) -> list:
    """from key_list return question list """

    question_list = []
    for key in key_list:
        question_list.extend(_question_selector(key))

    return _u(question_list)